// Copyright (c) 2015, AppFoundry. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

// TODO: Put public facing types in this file.

library d_calendar.base;

import "dart:html";
import "dart:core";
import "day.dart";

class Calendar
{ 
  Calendar(DivElement container)
  {  
    _generateHtml(container);
   
    _days = new Map();
    DateTime now = new DateTime.now(); 
    int weekday = now.weekday;
    
    for (int i = 1; i < 8; i++)
    {
      _currentWeek.add(now.add(new Duration(days:i-weekday)));     
    }

    for (int i = 0; i < 7; i++)
    {
      _divWeekdays[i].querySelector(".dayofweek").innerHtml = _weekdays[i];
    }    

    _render(0);
    _next.onClick.listen((_) => _render(1));
    _previous.onClick.listen((_) => _render(-1));
  }
     
  void _render(int week_offset) 
  {
    _cWeekOffset += week_offset;
    
    /// Update current week
    for (int i = 0; i <_currentWeek.length; i++)
    {
      _currentWeek[i] = _currentWeek[i].add(new Duration(days:7*week_offset));
    }

    for (int i = 0; i < 7; i++)
    {
      _divWeekdays[i].querySelector(".date").innerHtml = _currentWeek[i].day.toString();
      _divWeekdays[i].querySelector(".month").innerHtml = _months[_currentWeek[i].month - 1];
      _divWeekdays[i].querySelector(".content").children.clear();
      
      DateTime date = _currentWeek[i];      
      if (_days.containsKey(date.year))
      {        
        if (_days[date.year].containsKey(date.month))
        {
          Map<int, Day> week = _days[date.year][date.month];
          if (week.containsKey(date.day))
          {
            List<Element> children = _divWeekdays[i].querySelector(".content").children;
            children.add(week[_currentWeek[i].day].content);
          }
        }
      }
    }
    _week.innerHtml = calcWeekOfYear(_cWeekOffset).toString();
  }
  
  int calcWeekOfYear(int week_offset)
  {
    DateTime current = new DateTime.now().add(new Duration(days:7*week_offset));
    _year.innerHtml = current.year.toString();
    /// First day of year is not always monday! But we assume every week is 7 days. Offset the first week by number of days since monday.
    DateTime firstDayOfYear = new DateTime(current.year);     
    _daysPerMonth[1] = 31 + firstDayOfYear.weekday;    
    _daysPerMonth[2] = (current.year % 4 == 0 && current.year % 100 == 0 && current.year % 400 == 0) ? 29 : 28; /// Leap year
    _daysPerMonth[3] = 31;
    _daysPerMonth[4] = 30;
    _daysPerMonth[5] = 31;
    _daysPerMonth[6] = 30;
    _daysPerMonth[7] = 31;
    _daysPerMonth[8] = 31;
    _daysPerMonth[9] = 30;
    _daysPerMonth[10] = 31;
    _daysPerMonth[11] = 30;
    _daysPerMonth[12] = 31;
    
    int currentDayOfYear = 0;
    
    for (int i = 1; i < current.month; i++)
    {
      currentDayOfYear += _daysPerMonth[i];
    }
    currentDayOfYear += current.day;
    return (currentDayOfYear.toDouble() / 7).ceil();
  }
  
  Day getDay(int year, int month, int day)
  {  
    DateTime checkDate = new DateTime(year, month, day);
    if (checkDate.day != day || checkDate.year != year || checkDate.month != month) throw(new StateError("Date is not valid"));      
    if (!_days.containsKey(year)) _days[year] = new Map();     
    if (!_days[year].containsKey(month)) _days[year][month] = new Map();
    if (!_days[year][month].containsKey(day)) _days[year][month][day] = new Day(); 
    
    return _days[year][month][day];
  }   
  
  void _generateHtml(DivElement container)
  {
    if (container == null) throw new ArgumentError.notNull("container");
    
    DivElement buttonRow = new DivElement();    
    DivElement calendarRow = new DivElement();
    buttonRow.className = calendarRow.className = "row";
    
    _next = new ButtonElement();
    _previous = new ButtonElement();
    _previous.className = "small-1 small-offset-3 columns";
    _next.className = "small-1 columns end";
    _next.innerHtml = "&raquo;";
    _previous.innerHtml = "&laquo;";
    buttonRow.children.add(_previous);
    buttonRow.children.add(_next);
        
    DivElement monthInfo = new DivElement();
    monthInfo.className = "small-6 medium-3 large-1 columns weekday";
    _year = new HeadingElement.h3();    
    _week = new HeadingElement.h5();
    monthInfo.children.add(_year);
    monthInfo.children.add(_week);    
    calendarRow.children.add(monthInfo);
    
    for (int i = 0; i < 7; i++)
    {
      DivElement weekday = new DivElement();
      weekday.className = "small-6 medium-3 large-1 columns weekday";
      weekday.id = _weekdays[i];
      
      List<Element> children = new List();
      children.add(new DivElement());
      children.last.className = "dateinfo";
      children.add(new DivElement());
      children.last.className = "dayofweek";
      children.add(new DivElement());
      children.last.className = "date";
      children.add(new DivElement());
      children.last.className = "month";
      children.add(new HRElement());
      children.add(new DivElement());
      children.last.className = "content";
      
      children.forEach((Element e) => weekday.children.add(e));     
      _divWeekdays.add(weekday);
      calendarRow.children.add(weekday);
    }   
    calendarRow.children.last.classes.add("end");
    
    
    container.children.add(buttonRow);
    container.children.add(calendarRow);
  }
  
  List<DateTime> _currentWeek = new List();
  List<DivElement> _divWeekdays = new List();
  HeadingElement _year;
  HeadingElement _week;
  List<String> _weekdays = ["mon","tue","wed","thu","fri","sat","sun"];
  List<String> _months = ["januari","february","march","april","may","june","july","august","september","october","november","december"];
  Map<int, int> _daysPerMonth = new Map(); 
  ButtonElement _next;
  ButtonElement _previous;
  int _cWeekOffset = 0;
  Map<int, Map<int, Map<int, Day>>> _days; 
}